!***********************************************************************
! subroutine boxit                                                     *
!    Determine the corners of a single box which encloses all of the   *
! pixels with transverse field strength above a threshold. If there    *
! are no pixels above threshold, print a warning and do a potential    *
! field acute angle ambiguity resolution.                              *
!***********************************************************************

subroutine boxit(bthresh)

!-----------------------------------------------------------------------
   use sizes
   use bobs
   use bounds
   use flags
   use mgram_data
   use verbose

   implicit none

   integer :: i,j,nthresh
   real :: bthresh
!-----------------------------------------------------------------------
! --> Initialize.
   nxa=nx+1
   nxb=-1
   nya=ny+1
   nyb=-1
   nthresh=0
    
! --> Loop over pixels, checking which are above threshold. 

   do i=1,nx
      do j=1,ny

         if(bt(i,j).ge.bthresh) then
            nthresh=nthresh+1

            if(nxb.lt.i) nxb=i
            if(nxa.gt.i) nxa=i
            if(nyb.lt.j) nyb=j
            if(nya.gt.j) nya=j

         endif

      enddo
   enddo

! --> If no pixels are above threshold, print a warning, do a potential
! --> field acute angle ambiguity resolution, and exit.
   if(nthresh.eq.0) then
      write(*,*) '*** WARNING: NO PIXELS ABOVE THRESHOLD ***'
      write(*,*) '    check value of bthresh in par file'

      !call pacute(bthresh)

      if(irflag.eq.0) then
         call write_field()
      else if(irflag.eq.1) then
         call write_field()
         !call update_fits(filename)
      else if(irflag.eq.2) then
         call write_field()
      endif
      stop
   endif

   if(iverb.eq.2) write(*,*) 'box:',nxa,nxb,nya,nyb

end subroutine boxit
