c***********************************************************************
      subroutine nacute2(bthresh)
c***********************************************************************
c     In pixels where the magnitude of the transverse field is below a *
c     given threshold, resolve the 180 degree ambiguity by picking the *
c     direction closest to the average direction of the field at       *
c     neighboring pixels. The solution is iterated as the average may  *
c     change. Based on the approach in UHIM code.                      *
c***********************************************************************
      include "include_file.sizes.f"
      include "include_file.field.f"
      include "include_file.bobs.f"
c-----------------------------------------------------------------------
      integer i,j,i1d,j1d,iter,niter,nflip,nflip1,indx(nxmax*nymax)
      real Bxs,Bys,bdotb,bthresh
      real,dimension(nxmax*nymax) :: bt1d,ba1d
      equivalence(bt,bt1d)
      equivalence(ba,ba1d)
c-----------------------------------------------------------------------
c --> Create an index array for pixels sorted on transverse field.

      call sortrx(nxmax*nymax,-bt1d,indx)

c --> Maximum number of iterations. 
      niter=50

      nflip=0
      nflip1=1
      iter=1
      dowhile(nflip1.gt.0.and.iter.lt.niter)

c --> This is a local test, so loop over pixels and apply at each one. 

         nflip1=0
         do 2 i1d=1,nx*ny
            j1d=indx(i1d)

c --> 2-d indices for this pixel.
            j=1+j1d/nxmax
            i=j1d-(j-1)*nymax

c --> Only apply to points below threshold.

            if(bt1d(j1d).le.bthresh) then

c --> Compute dot product of transverse field at this pixel with
c --> transverse field at neighboring pixels. 

c --> If edge pixel, has fewer neighbors.
               if(i.eq.1) then
                  if(j.eq.1) then
                     Bxs=Bx(i+1,j+1)+Bx(i,j+1)+Bx(i+1,j)
                     Bys=By(i+1,j+1)+By(i,j+1)+By(i+1,j)
                  elseif(j.eq.ny) then
                     Bxs=Bx(i+1,j-1)+Bx(i,j-1)+Bx(i+1,j)
                     Bys=By(i+1,j-1)+By(i,j-1)+By(i+1,j)
                  else
                     Bxs=Bx(i,j-1)+Bx(i+1,j-1)+Bx(i+1,j)
     +                  +Bx(i,j+1)+Bx(i+1,j+1)
                     Bys=By(i,j-1)+By(i+1,j-1)+By(i+1,j)
     +                  +By(i,j+1)+By(i+1,j+1)
                  endif
               elseif(i.eq.nx) then
                  if(j.eq.1) then
                     Bxs=Bx(i-1,j+1)+Bx(i,j+1)+Bx(i-1,j)
                     Bys=By(i-1,j+1)+By(i,j+1)+By(i-1,j)
                  elseif(j.eq.ny) then
                     Bxs=Bx(i-1,j-1)+Bx(i,j-1)+Bx(i-1,j)
                     Bys=By(i-1,j-1)+By(i,j-1)+By(i-1,j)
                  else
                     Bxs=Bx(i,j-1)+Bx(i-1,j-1)+Bx(i-1,j)
     +                  +Bx(i,j+1)+Bx(i-1,j+1)
                     Bys=By(i,j-1)+By(i-1,j-1)+By(i-1,j)
     +                  +By(i,j+1)+By(i-1,j+1)
                  endif
               else
                  if(j.eq.1) then
                     Bxs=Bx(i-1,j+1)+Bx(i-1,j)+Bx(i,j+1)
     +                  +Bx(i+1,j+1)+Bx(i+1,j)
                     Bys=By(i-1,j+1)+By(i-1,j)+By(i,j+1)
     +                  +By(i+1,j+1)+By(i+1,j)
                  elseif(j.eq.ny) then
                     Bxs=Bx(i-1,j-1)+Bx(i-1,j)+Bx(i,j-1)
     +                  +Bx(i+1,j-1)+Bx(i+1,j)
                     Bys=By(i-1,j-1)+By(i-1,j)+By(i,j-1)
     +                  +By(i+1,j-1)+By(i+1,j)
                  else
                     Bxs=Bx(i-1,j-1)+Bx(i,j-1)+Bx(i+1,j-1)
     +                  +Bx(i-1,j+1)+Bx(i,j+1)+Bx(i+1,j+1)
     +                  +Bx(i-1,j)+Bx(i+1,j)
                     Bys=By(i-1,j-1)+By(i,j-1)+By(i+1,j-1)
     +                  +By(i-1,j+1)+By(i,j+1)+By(i+1,j+1)
     +                  +By(i-1,j)+By(i+1,j)
                  endif
               endif

               bdotb=Bxs*Bx(i,j)+Bys*By(i,j)

c --> Flip direction of transverse field if dot product is negative.
               if(bdotb.lt.0.) then
                  nflip1=nflip1+1
                  Bx(i,j)=-Bx(i,j)
                  By(i,j)=-By(i,j)
               endif

            endif

    1       continue
    2    continue
         iter=iter+1
         nflip=nflip+nflip1
         write(*,*) 'iteration',iter,nflip1,nflip
      enddo
      write(*,*) 'total number of flips',nflip
      write(*,*) 'fraction flipped',float(nflip)/float(nx*ny)

      return
      end
