!***********************************************************************
! subroutine rd_field                                                  *
!    Reads in magnetic field data plus pointing information.           *
!***********************************************************************

subroutine rd_field(filename)

!-----------------------------------------------------------------------
   use sizes
   use bobs
   use constant
   use disk_center
   use ephemeris
   use flags
   use mgram_data
   use pix_size
   use point
   use spherical_position

   implicit none

   integer :: i,j
integer,dimension(1) :: seed
   real :: phase,dtor,xx,yy,y2
real :: rnd
   character(len=256) :: filename
!-----------------------------------------------------------------------
   open(2,file=filename,status='old')

! --> Read in array dimensions.

   read(2,*) nx,ny

! --> Allocate memory.
   allocate(bl(nx,ny),bt(nx,ny),ba(nx,ny))
   allocate(Bx(nx,ny),By(nx,ny),Bz(nx,ny),dBzdz(nx,ny))

! --> Read in pixel size.

   read(2,*) dxi,dyi

! --> Read in ephemeris information:
! --> b=solar b-angle
! --> p=solar p-angle
! --> radius=solar radius

   read(2,*) b,p0,radius

! --> Check whether conversion to degrees is needed for ephemeris.
   if(ipunit.eq.1) then
      dtor=pi/180.
      b=b*dtor
      p0=p0*dtor
   endif

! --> Read in pointing information:
! --> theta=central meridian angle
! --> phi=latitude

   read(2,*) theta,phi

! --> Check whether conversion to degrees is needed for pointing.
   if(ipunit.eq.1) then
      theta=theta*dtor
      phi=phi*dtor
   endif

! --> If working in spherical geometry, generate colat/lon for each pixel.
   if(igflag.eq.2) call colatlon

! --> Read in line of sight field.
   do j=1,ny
      read(2,*) (bl(i,j),i=1,nx)
   enddo

! --> Read in transverse field.
   do j=1,ny
      read(2,*) (bt(i,j),i=1,nx)
   enddo

! --> Read in azimuthal angle.
! --> Check whether conversion to degrees is needed for azimuth.
   if(iaunit.eq.1) then
      dtor=pi/180.
      do j=1,ny
         read(2,*) (ba(i,j),i=1,nx)
         do i=1,nx
            ba(i,j)=ba(i,j)*dtor
         enddo
      enddo
   else
      do j=1,ny
         read(2,*) (ba(i,j),i=1,nx)
      enddo
   endif

! --> Randomize initial azimuth.
   write(*,*) 'randomizing initial azimuth'
   seed(1)=1234
   call random_seed(put=seed)
   do i=1,nx
      do j=1,ny
         call random_number(rnd)
         if(rnd.gt.0.5.and.bt(i,j).gt.0.) ba(i,j)=ba(i,j)+pi
      enddo
   enddo

! --> Calculate Cartesian (image) components.
   phase=0.5*float(iaflag)*pi
   Bx=bt*cos(ba+phase)
   By=bt*sin(ba+phase)
   Bz=bl

end subroutine rd_field
