!***********************************************************************
! subroutine rd_param                                                  *
!    Read in some parameters which set options for the code.           *
!***********************************************************************

subroutine rd_param(parfile, filename,athresh,bthresh)

!-----------------------------------------------------------------------
   use sizes
   use anneal
   use flags
   use fits
   use maskvec
   use ntiles
   use pad
   use ranseed
   use verbose
   use WeightingFactor

   implicit none

   integer :: iseed
   real :: athresh,bthresh
   character(len=256),intent(in) :: parfile
   character(len=256),intent(out) :: filename
!-----------------------------------------------------------------------
   open(1, file=parfile, status='old')


! --> filename for the input file.
   read(1,'(a256)') filename

! --> irflag determines format of input: 1: FITS, 0: arrays
! --> ibflag determines whether field is specified as
! -->    0: two components and an angle
! -->    1: magnitude and two angles
! --> iaflag determines the direction of zero azimuth as
! -->    0: west
! -->    1: north (UHIM standard)
! --> igflag determines whether to use spherical or planar geometry
! -->    1: planar geometry
! -->    2: spherical geometry
   read(1,*) irflag,ibflag,iaflag,igflag,ipflag

! --> npad determines whether to surround with zeros.
! --> nap sets the number of pixels over which to apodize the edge.
! --> ntx, nty set the number of tiles which cover the full sphere for
! --> generating the potential field in spherical geometry. 
   read(1,*) npad,nap,ntx,nty

   if(nap.gt.npad) nap=npad
   if(ntx.ne.nty) then
      write(*,*) 'WARNING: using non-square tiles',ntx,nty
   endif

! --> athresh determines the level above which to anneal.
! --> bthresh determines the level below which to use acute angle.
! --> nerode determines the number of pixels by which to erode the mask.
! --> ngrow determines the number of pixels by which to grow the mask.
   read(1,*) athresh,bthresh,nerode,ngrow

! --> iaunit specifies whether azimuth is in radians or degrees. 
! --> ipunit specifies whether pointing is in radians or degrees. 
! --> incflag specifies whether inclination of 0 is vertical or
! --> horizontal.
   read(1,*) iaunit,ipunit,incflag

! --> The following control the annealing schedule and similar.

   read(1,*) iseed,iverb,neq
   read(1,*) lambda,tfac0,tfactr

! --> seed for random number generator

   seed=iseed
   if (seed.le.0) seed=123456

! --> check other input parameters

   if (lambda.lt.0) then
      print*,'read_inputs: lambda<0'
      stop
   endif
   if (tfac0.lt.0) then
      print*,'read_inputs: tfac0<0'
      stop
   endif
   if (neq.le.0) then
      print*,'read_inputs: neq.le.0'
      stop
   endif
   if ((tfactr.lt.0).or.(tfactr.gt.1)) then
      print*,'read_inputs: tfactr problem'
      stop
   endif

! --> If the data are read from a FITS file, need some additional 
! --> information about how to extract the necessary data.

   if(irflag.eq.1) then

! --> Planes in the FITS image containing the longitudinal and
! --> transverse field components, the azimuth, and the fill fraction.
      read(1,*) iblong,ibtrans,ibazim,ibfill
 
! --> Keyword names for longitude, latitude, p and b angles, and pixel
! --> size.
      read(1,'(a8)') cmdkey
      read(1,'(a8)') latkey
      read(1,'(a8)') xcenkey
      read(1,'(a8)') ycenkey
      read(1,'(a8)') pkey
      read(1,'(a8)') bkey
      read(1,'(a8)') radkey
      read(1,'(a8)') pxkey
      read(1,'(a8)') pykey

   else if(irflag.eq.2) then
! --> If the FITS file comes from the Hinode SOT/SP pipeline: 
! --> number of extension to FITS file containing field strength,
! --> inclination and azimuth, and the continuum intensity.
      read(1,*) jfs,jfi,jfa,jsf,jci
   endif

end subroutine rd_param
