!***********************************************************************************************************************************
!===================================================================================================================================
! subroutine setup_mask
! This subroutine sets up the masking array. Pixels with mask.lt.1 are considered fixed by the subroutine minimise_energy.
!
! 2009,2010, Ashley Crouch, ash@cora.nwra.com
! re-written by Graham Barnes, graham@cora.nwra.com
!===================================================================================================================================

subroutine setup_mask(athresh,bthresh)

   use sizes
   use bobs
   use bounds
   use flags
   use maskvec
   use spherical_position
   use verbose

   implicit none

   integer :: i,j
   integer(kind=1),dimension(:,:),allocatable :: temp
   real :: athresh,bthresh,y2
   real,dimension(:,:),allocatable :: rdist
!-----------------------------------------------------------------------
! --> Allocate memory for the mask array.
!
   allocate(mask(nx,ny),temp(nx,ny))
!
! --> Initialize mask array.
!
   mask=1
!
! This loop sets mask=0 in above threshold pixels. For spherical
! geometry, the threshold is adjusted based on the distance from disk
! center.
!
   if(igflag.eq.1) then
      do j=nya,nyb
         do i=nxa,nxb
            if(bt(i,j).ge.bthresh) mask(i,j)=0
         enddo
      enddo
   elseif(igflag.eq.2) then
      allocate(rdist(nx,ny))
      do j=nya,nyb
         y2=cosp(j)*cosp(j)
         do i=nxa,nxb
            !rdist(i,j)=sqrt(1.-sint(i,j)*sint(i,j)-y2*cost(i,j)*cost(i,j))
            rdist(i,j)=sint(i,j)*sint(i,j)+y2*cost(i,j)*cost(i,j)
            if(bt(i,j).ge.bthresh*(2.-sqrt(1.-rdist(i,j)))) mask(i,j)=0
         enddo
      enddo
   endif
!
! --> Erode the mask by nerode pixels (equivalent to growing the inverse
! --> of the mask).
!
   call grown(nerode)
!
! --> Invert the mask for growing.
!
   do i=nxa,nxb
      do j=nya,nyb
         mask(i,j)=1-mask(i,j)
      enddo
   enddo
   temp=mask
!
! --> Grow the mask by ngrow pixels.
!
   call grown(ngrow)
!
! --> Add temp to mask to correspond to values:
! --> mask=2: use annealing result
! --> mask=1: use smoothing result
! --> mask=0: no result
!
   mask=mask+temp
!
! --> Make sure that pixels along the edges are excluded from the mask.
!
   do i=nxa,nxb
      mask(i,nya)=0
      mask(i,nyb)=0
   enddo
   do j=nya,nyb
      mask(nxa,j)=0
      mask(nxb,j)=0
   enddo
!
! --> Make sure pixels off the disk are excluded from the mask, then
! --> deallocate memory.
!
   if(igflag.eq.2) then
      do i=1,nx
         do j=1,ny
            if(rdist(i,j).ge.1.) mask(i,j)=0
         enddo
      enddo
      deallocate(rdist)
   endif
!
! --> Deallocate memory for temp array.
!
   deallocate(temp)

end subroutine setup_mask
!***********************************************************************************************************************************
