!***********************************************************************
! subroutine update_fits                                               *
!    Overwrites the azimuth angle in a FITS file. Adds a keyword       *
! comment that the ambiguity has been resolved using ME0.              *
!***********************************************************************

subroutine update_fits(filename)

!-----------------------------------------------------------------------
   use sizes
   use bobs
   use constant
   use ephemeris
   use fits
   use flags
   use mgram_data

   implicit none

   integer :: status,unit,readwrite,blocksize
   integer :: fpixels(3),lpixels(3),group,i,j
   real :: rtod,phase
   real,dimension(:),allocatable :: ba1d
   character(len=256) :: filename
   character(len=80) :: comment
!-----------------------------------------------------------------------
! --> Allocate memory for 1-d array.
   allocate(ba1d(nx*ny))

! --> The status parameter must always be initialized.
   status=0

! --> Get an unused Logical Unit Number to use to open the FITS file.
   call ftgiou(unit,status)

! --> Open the FITS file, with read-write access.  The returned blocksize
! --> parameter is obsolete and should be ignored. 
   readwrite=1
   call ftopen(unit,filename,readwrite,blocksize,status)

! --> Recompute azimuth from ambiguity-resolved Cartesian components 
! --> unless transverse field is zero, in which case use original angle.

   phase=0.5*float(iaflag)*pi
   do i=1,nx
      do j=1,ny
         if(bt(i,j).ne.0.) then
            ba1d(i+(j-1)*nx)=atan2(By(i,j),Bx(i,j))-phase
         else
            ba1d(i+(j-1)*nx)=ba(i,j)
         endif
      enddo
   enddo

! --> Convert back to degrees, if necessary.
   if(iaunit.eq.1) then
      rtod=180./pi
      ba1d=ba1d*rtod
   endif

! --> Initialize variables for writing azimuth. 
   group=1

! --> First and last pixels in each array dimension.
   fpixels(1)=1
   fpixels(2)=1
   fpixels(3)=ibazim
   lpixels(1)=naxes(1)
   lpixels(2)=naxes(2)
   lpixels(3)=ibazim

! --> Write new azimuthal angle.

   call ftpsse(unit,group,3,naxes,fpixels,lpixels,ba1d,status)

! --> Append a comment that the ambiguity has been resolved using ME0.

   comment='Ambiguity has been resolved using ME0 code.'

   call ftpcom(unit,comment,status)

! --> Deallocate memory for 1-d array.
   deallocate(ba1d)

! --> The FITS file must always be closed before exiting the program. 
! --> Any unit numbers allocated with FTGIOU must be freed with FTFIOU.
   call ftclos(unit, status)
   call ftfiou(unit, status)

! --> Check for any error, and if so print out error messages.
! --> The PRINTERROR subroutine is listed near the end of this file.
   if(status.gt.0) call printerror(status)

end subroutine update_fits
